@@ -2,6 +2,7 @@
  * crypto.c
  *
  * Copyright (C) AB Strakt 2001, All rights reserved
+ * Copyright (C) Jean-Paul Calderone 2008, All rights reserved
  *
  * Main file of crypto sub module.
  * See the file RATIONALE for a short explanation of why this module was written.
@@ -17,8 +18,6 @@ Main file of crypto sub module.\n\
 See the file RATIONALE for a short explanation of why this module was written.\n\
 ";
 
-static char *CVSid = "@(#) $Id: crypto.c,v 1.28 2004/08/09 14:56:05 martin Exp $";
-
 void **ssl_API;
 
 PyObject *crypto_Error;
@@ -584,10 +583,16 @@ Returns:   The PKey object\n\
 static PyObject *
 crypto_PKey(PyObject *spam, PyObject *args)
 {
+    crypto_PKeyObj *py_pkey;
+
     if (!PyArg_ParseTuple(args, ":PKey"))
         return NULL;
 
-    return (PyObject *)crypto_PKey_New(EVP_PKEY_new(), 1);
+    py_pkey = crypto_PKey_New(EVP_PKEY_new(), 1);
+    if (py_pkey) {
+	py_pkey->initialized = 0;
+    }
+    return py_pkey;
 }
 
 static char crypto_X509Extension_doc[] = "\n\
@@ -647,6 +652,26 @@ crypto_NetscapeSPKI(PyObject *spam, PyObject *args)
     return (PyObject *)crypto_NetscapeSPKI_New(spki, 1);
 }
 
+static char crypto_X509_verify_cert_error_string_doc[] = "\n\
+Get X509 verify certificate error string.\n\
+\n\
+Arguments: errnum - Error number\n\
+Returns:   Error string as a Python string\n\
+";
+
+static PyObject *
+crypto_X509_verify_cert_error_string(PyObject *spam, PyObject *args)
+{
+    int errnum;
+    const char *str;
+
+    if (!PyArg_ParseTuple(args, "i", &errnum))
+        return NULL;
+
+    str = X509_verify_cert_error_string(errnum);
+    return PyString_FromString(str);
+}
+
 /* Methods in the OpenSSL.crypto module (i.e. none) */
 static PyMethodDef crypto_methods[] = {
     /* Module functions */
@@ -665,6 +690,7 @@ static PyMethodDef crypto_methods[] = {
     { "PKey",    (PyCFunction)crypto_PKey,    METH_VARARGS, crypto_PKey_doc },
     { "X509Extension", (PyCFunction)crypto_X509Extension, METH_VARARGS, crypto_X509Extension_doc },
     { "NetscapeSPKI", (PyCFunction)crypto_NetscapeSPKI, METH_VARARGS, crypto_NetscapeSPKI_doc },
+    { "X509_verify_cert_error_string", (PyCFunction)crypto_X509_verify_cert_error_string, METH_VARARGS, crypto_X509_verify_cert_error_string_doc },
     { NULL, NULL }
 };
 
